@@ -22,6 +22,7 @@
  */
 
 const Gdk = imports.gi.Gdk;
+const GdkPixbuf = imports.gi.GdkPixbuf;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const GObject = imports.gi.GObject;
@@ -33,6 +34,7 @@ const Mainloop = imports.mainloop;
 const Application = imports.application;
 const MapView = imports.mapView;
 const SearchPopup = imports.searchPopup;
+const ContextMenu = imports.contextMenu;
 const Utils = imports.utils;
 const Config = imports.config;
 
@@ -42,9 +44,12 @@ const _CONFIGURE_ID_TIMEOUT = 100; // msecs
 const _WINDOW_MIN_WIDTH = 600;
 const _WINDOW_MIN_HEIGHT = 500;
 
+const _PLACE_ICON_SIZE = 20;
+
 const SearchResults = {
-    COL_DESCRIPTION:  0,
-    COL_LOCATION:     1
+    COL_ICON:         0,
+    COL_DESCRIPTION:  1,
+    COL_LOCATION:     2
 };
 
 const MainWindow = new Lang.Class({
@@ -54,10 +59,8 @@ const MainWindow = new Lang.Class({
         this._configureId = 0;
         let ui = Utils.getUIObject('main-window', [ 'app-window',
                                                     'window-content',
-                                                    'search-entry',
-                                                    'track-user-button']);
-        let grid = ui.windowContent,
-            toggle = ui.trackUserButton;
+                                                    'search-entry' ]);
+        let grid = ui.windowContent;
         this._searchEntry = ui.searchEntry;
         this.window = ui.appWindow;
         this.window.application = app;
@@ -68,6 +71,8 @@ const MainWindow = new Lang.Class({
 
         this.mapView.gotoUserLocation(false);
 
+        this._contextMenu = new ContextMenu.ContextMenu(this.mapView);
+
         this._initSearchWidgets();
         this._initActions();
         this._initSignals();
@@ -81,16 +86,17 @@ const MainWindow = new Lang.Class({
     },
 
     _initSearchWidgets: function() {
-        this._searchPopup = new SearchPopup.SearchPopup();
+        this._searchPopup = new SearchPopup.SearchPopup(10);
 
         let model = new Gtk.ListStore();
-        model.set_column_types([GObject.TYPE_STRING,
+        model.set_column_types([GdkPixbuf.Pixbuf,
+                                GObject.TYPE_STRING,
                                 GObject.TYPE_OBJECT]);
         this._searchPopup.setModel(model);
         this._searchPopup.connect('selected',
                                   this._onSearchPopupSelected.bind(this));
         this.mapView.view.connect('button-press-event',
-                             this._searchPopup.hide.bind(this._searchPopup));
+                                  this._searchPopup.hide.bind(this._searchPopup));
         this._searchEntry.connect('changed',
                                   this._searchPopup.hide.bind(this._searchPopup));
     },
@@ -220,7 +226,7 @@ const MainWindow = new Lang.Class({
         let model = this._searchPopup.getModel();
         let location = model.get_value(iter, SearchResults.COL_LOCATION);
 
-        this.mapView.showLocation(location);
+        this.mapView.showNGotoLocation(location);
         this._searchPopup.hide();
     },
 
@@ -228,31 +234,64 @@ const MainWindow = new Lang.Class({
         let searchString = this._searchEntry.get_text();
 
         if (searchString.length > 0) {
+            let model = this._searchPopup.getModel();
+
+            model.clear();
+            this._searchPopup.showSpinner();
             this.mapView.geocodeSearch(searchString,
                                        this._showSearchResults.bind(this));
         }
     },
 
+    // We want to match case insensitive but present in the correct case.
+    _boldMatch: function(description, searchStringLower) {
+        let index = description.toLowerCase().indexOf(searchStringLower);
+
+        if (index !== -1) {
+            let substring = description.substring(index,
+                                                  index + searchStringLower.length);
+
+            description = description.replace(substring, substring.bold());
+        }
+
+        return description;
+    },
+
     _showSearchResults: function(places) {
         let model = this._searchPopup.getModel();
 
-        model.clear();
-        places.forEach(function(place) {
+        if (places === null) {
+            this._searchPopup.hide();
+            return;
+        }
+
+        // Lower case to match case insensitive
+        let searchStringLower = this._searchEntry.text.toLowerCase();
+
+        places.forEach((function(place) {
             let iter = model.append();
             let location = place.get_location();
+            let icon = place.icon;
 
             if (location == null)
                 return;
 
-            let description_markup = '<b>' + location.description + '</b>';
+            let description = GLib.markup_escape_text(location.description, -1);
+            description = this._boldMatch(description, searchStringLower);
+
             model.set(iter,
                       [SearchResults.COL_DESCRIPTION,
                        SearchResults.COL_LOCATION],
-                      [description_markup,
+                      [description,
                        location]);
-        });
-        this._searchPopup.setModel(model);
-        this._searchPopup.show();
+
+            if (icon !== null) {
+                Utils.load_icon(icon, _PLACE_ICON_SIZE, function(pixbuf) {
+                    model.set(iter, [SearchResults.COL_ICON], [pixbuf]);
+                });
+            }
+        }).bind(this));
+        this._searchPopup.showResult();
     },
 
     _quit: function() {
@@ -269,7 +308,15 @@ const MainWindow = new Lang.Class({
     },
 
     _onGotoUserLocationActivate: function() {
-        this.mapView.gotoUserLocation(true);
+        if (this.mapView.geoclue.userSetLocation) {
+            Utils.once(this.mapView.geoclue,
+                       'location-changed',
+                       (function() {
+                this.mapView.gotoUserLocation(true);
+            }).bind(this));
+            this.mapView.geoclue.findLocation();
+        } else
+            this.mapView.gotoUserLocation(true);
     },
 
     _onMapTypeMenuActivate: function(action) {
@@ -293,8 +340,8 @@ const MainWindow = new Lang.Class({
             program_name: _("Maps"),
             comments: _("A map application for GNOME"),
             copyright: 'Copyright ' + String.fromCharCode(0x00A9) +
-                ' 2011' + String.fromCharCode(0x2013) +
-                '2013 Red Hat, Inc.',
+                       ' 2011' + String.fromCharCode(0x2013) +
+                       '2013 Red Hat, Inc.',
             license_type: Gtk.License.GPL_2_0,
             logo_icon_name: 'gnome-maps',
             version: Config.PACKAGE_VERSION,
