@@ -33,6 +33,7 @@ const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Signals = imports.signals;
 
+const Application = imports.application;
 const ZoomControl = imports.zoomControl;
 const Sidebar = imports.sidebar;
 const Utils = imports.utils;
@@ -82,10 +83,10 @@ const MapView = new Lang.Class({
         this._zoomControl = new ZoomControl.ZoomControl(this);
         overlay.add_overlay(this._zoomControl);
 
-        this._geoclue = new Geoclue.Geoclue();
+        this.geoclue = new Geoclue.Geoclue();
         this._updateUserLocation();
-        this._geoclue.connect("location-changed",
-                              this._updateUserLocation.bind(this));
+        this.geoclue.connect("location-changed",
+                             this._updateUserLocation.bind(this));
     },
 
     setMapType: function(mapType) {
@@ -96,16 +97,17 @@ const MapView = new Lang.Class({
     geocodeSearch: function(searchString, searchCompleteCallback) {
         let forward = Geocode.Forward.new_for_string(searchString);
         let places = [];
+        let answerCount =
+            Application.settings.get_value('max-search-results').get_int32();
 
-        this._lastSearch = searchString;
+        forward.set_answer_count(answerCount);
         forward.search_async (null, (function(forward, res) {
             try {
                 places = forward.search_finish(res);
             } catch (e) {
                 places = null;
             }
-            if (places !== null)
-                searchCompleteCallback(places);
+            searchCompleteCallback(places);
         }).bind(this));
     },
 
@@ -139,10 +141,10 @@ const MapView = new Lang.Class({
     },
 
     _updateUserLocation: function() {
-        if (!this._geoclue.location)
+        if (!this.geoclue.location)
             return;
 
-        this._userLocation = new UserLocation.UserLocation(this._geoclue.location, this);
+        this._userLocation = new UserLocation.UserLocation(this.geoclue.location, this);
         this._userLocation.show(this._userLocationLayer);
         this.emit('user-location-changed');
     },
@@ -152,6 +154,12 @@ const MapView = new Lang.Class({
         let mapLocation = new MapLocation.MapLocation(location, this);
 
         mapLocation.show(this._markerLayer);
+
+        return mapLocation;
+    },
+
+    showNGotoLocation: function(location) {
+        let mapLocation = this.showLocation(location);
         mapLocation.goTo(true);
     },
 
